/*	This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

package com.nonoo.audiorecorder;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.TargetDataLine;

public class AudioRecorder implements Runnable {

	// note that this is declared as static so if we stop/close it, the thread will stop
	private static TargetDataLine tdl;
	private File outputFile;

	AudioRecorder(final TargetDataLine tdl, final File outputFile) {
		AudioRecorder.tdl = tdl;
		this.outputFile = outputFile;
	}

	@Override
	public void run() {
		tdl.start();
		try {
			// this will block until tdl doesn't get closed
			AudioSystem.write(new AudioInputStream(tdl), AudioFileFormat.Type.WAVE, outputFile);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void stopRecording() {
		tdl.stop();
		tdl.close();
	}
	
	public static void main(String[] args) {
		File outputFile = new File("output.wav");

		AudioFormat audioFormat = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, 44100.0F, 16, 2, 4, 44100.0F, false);
		DataLine.Info info = new DataLine.Info(TargetDataLine.class, audioFormat);

		TargetDataLine targetDataLine = null;
		try {
			targetDataLine = (TargetDataLine) AudioSystem.getLine(info);
			targetDataLine.open(audioFormat);
		} catch (LineUnavailableException e1) {
			e1.printStackTrace();
		}

		// creating the recorder thread from this class' instance
		AudioRecorder audioRecorder = new AudioRecorder(targetDataLine, outputFile);
		Thread audioRecorderThread = new Thread(audioRecorder);

		// we use this to read line from the standard input
		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		System.out.println("Press ENTER to start recording!");
		try {
			br.readLine();
		} catch (IOException e) {
			e.printStackTrace();
		}

		audioRecorderThread.start();

		System.out.println("Recording... press ENTER to stop recording!");
		try {
			br.readLine();
		} catch (IOException e) {
			e.printStackTrace();
		}

		audioRecorder.stopRecording();
		
		try {
			// waiting for the recorder thread to stop
			audioRecorderThread.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		System.out.println("Recording stopped.");
	}
}
